@@ -1,5 +1,5 @@
-/* Copyright 2017 - 2023 R. Thomas
- * Copyright 2017 - 2023 Quarkslab
+/* Copyright 2017 - 2024 R. Thomas
+ * Copyright 2017 - 2024 Quarkslab
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -20,21 +20,10 @@
 #include "LIEF/PE/enums.h"
 
 
-extern "C"
-{
+extern "C" {
 
 const char* PE_TYPES_to_string(enum LIEF_PE_PE_TYPES e) {
   return LIEF::PE::to_string(static_cast<LIEF::PE::PE_TYPE>(e));
 }
 
-
-const char* MACHINE_TYPES_to_string(enum LIEF_PE_MACHINE_TYPES e) {
-  return LIEF::PE::to_string(static_cast<LIEF::PE::MACHINE_TYPES>(e));
-}
-
-
-const char* SUBSYSTEM_to_string(enum LIEF_PE_SUBSYSTEM e) {
-  return LIEF::PE::to_string(static_cast<LIEF::PE::SUBSYSTEM>(e));
-}
-
 }
