@@ -1,5 +1,5 @@
-/* Copyright 2017 - 2023 R. Thomas
- * Copyright 2017 - 2023 Quarkslab
+/* Copyright 2017 - 2024 R. Thomas
+ * Copyright 2017 - 2024 Quarkslab
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -13,19 +13,15 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef PY_LIEF_ANDROID_H_
-#define PY_LIEF_ANDROID_H_
-
-#include "LIEF/platforms/android.hpp"
-#include "pyLIEF.hpp"
-
-namespace LIEF {
-namespace Android {
-
-void init_python_module(py::module&);
-void init_versions(py::module&);
+#ifndef PY_LIEF_ANDROID_H
+#define PY_LIEF_ANDROID_H
 
+namespace nanobind {
+class module_;
 }
+
+namespace LIEF::Android::py {
+void init_module(nanobind::module_&);
 }
 
 #endif
