@@ -1,5 +1,5 @@
-/* Copyright 2017 - 2023 R. Thomas
- * Copyright 2017 - 2023 Quarkslab
+/* Copyright 2017 - 2024 R. Thomas
+ * Copyright 2017 - 2024 Quarkslab
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -34,7 +34,6 @@
 #include "LIEF/MachO/CodeSignatureDir.hpp"
 #include "LIEF/MachO/LinkerOptHint.hpp"
 #include "LIEF/MachO/TwoLevelHints.hpp"
-#include "LIEF/MachO/EnumToString.hpp"
 #include "LIEF/utils.hpp"
 
 #include "Object.tcc"
@@ -50,9 +49,14 @@
 namespace LIEF {
 namespace MachO {
 
-inline result<MACHO_TYPES> magic_from_stream(BinaryStream& stream) {
-  stream.setpos(0);
-  if (auto magic_res = stream.read<uint32_t>()) {
+inline result<MACHO_TYPES> magic_from_stream(BinaryStream& stream,
+                                             bool keep_offset = false) {
+  ScopedStream scoped(stream);
+  if (!keep_offset) {
+    scoped->setpos(0);
+  }
+
+  if (auto magic_res = scoped->read<uint32_t>()) {
     return static_cast<MACHO_TYPES>(*magic_res);
   }
   return make_error_code(lief_errors::read_error);
@@ -66,7 +70,8 @@ bool is_macho(BinaryStream& stream) {
             magic == MACHO_TYPES::MH_MAGIC_64 ||
             magic == MACHO_TYPES::MH_CIGAM_64 ||
             magic == MACHO_TYPES::FAT_MAGIC ||
-            magic == MACHO_TYPES::FAT_CIGAM);
+            magic == MACHO_TYPES::FAT_CIGAM ||
+            magic == MACHO_TYPES::NEURAL_MODEL);
   }
   return false;
 }
@@ -96,17 +101,22 @@ bool is_fat(const std::string& file) {
   return false;
 }
 
-bool is_64(const std::string& file) {
-  if (auto stream = FileStream::from_file(file)) {
-    if (auto magic_res = magic_from_stream(*stream)) {
-      const MACHO_TYPES magic = *magic_res;
-      return magic == MACHO_TYPES::MH_MAGIC_64 ||
-             magic == MACHO_TYPES::MH_CIGAM_64;
-    }
+bool is_64(BinaryStream& stream) {
+  ScopedStream scoped(stream, 0);
+  if (auto magic_res = magic_from_stream(*scoped)) {
+    const MACHO_TYPES magic = *magic_res;
+    return magic == MACHO_TYPES::MH_MAGIC_64 ||
+           magic == MACHO_TYPES::MH_CIGAM_64;
   }
   return false;
 }
 
+bool is_64(const std::string& file) {
+  if (auto stream = FileStream::from_file(file)) {
+    return is_64(*stream);
+  }
+  return false;
+}
 
 bool check_layout(const FatBinary& fat, std::string* error) {
   bool is_ok = true;
@@ -194,17 +204,15 @@ bool check_valid_paths(const Binary& binary, std::string* error) {
   int dependents_count  = 0;
   for (const LoadCommand& cmd : binary.commands()) {
     switch (cmd.command()) {
-      case LOAD_COMMAND_TYPES::LC_ID_DYLIB:
+      case LoadCommand::TYPE::ID_DYLIB:
         {
           has_install_name = true;
-          /*
-           * Fallback
-           */
+          [[fallthrough]];
         }
-      case LOAD_COMMAND_TYPES::LC_LOAD_DYLIB:
-      case LOAD_COMMAND_TYPES::LC_LOAD_WEAK_DYLIB:
-      case LOAD_COMMAND_TYPES::LC_REEXPORT_DYLIB:
-      case LOAD_COMMAND_TYPES::LC_LOAD_UPWARD_DYLIB:
+      case LoadCommand::TYPE::LOAD_DYLIB:
+      case LoadCommand::TYPE::LOAD_WEAK_DYLIB:
+      case LoadCommand::TYPE::REEXPORT_DYLIB:
+      case LoadCommand::TYPE::LOAD_UPWARD_DYLIB:
         {
           if (!DylibCommand::classof(&cmd)) {
             LIEF_ERR("{} is not associated with a DylibCommand which should be the case",
@@ -212,7 +220,7 @@ bool check_valid_paths(const Binary& binary, std::string* error) {
             break;
           }
           auto& dylib = *cmd.as<DylibCommand>();
-          if (dylib.command() != LOAD_COMMAND_TYPES::LC_ID_DYLIB) {
+          if (dylib.command() != LoadCommand::TYPE::ID_DYLIB) {
             ++dependents_count;
           }
           break;
@@ -221,8 +229,8 @@ bool check_valid_paths(const Binary& binary, std::string* error) {
     }
   }
 
-  const FILE_TYPES ftype = binary.header().file_type();
-  if (ftype == FILE_TYPES::MH_DYLIB) {
+  const Header::FILE_TYPE ftype = binary.header().file_type();
+  if (ftype == Header::FILE_TYPE::DYLIB) {
     if (!has_install_name) {
       if (error) {
         *error = fmt::format(R"delim(
@@ -241,7 +249,8 @@ bool check_valid_paths(const Binary& binary, std::string* error) {
       return false;
     }
   }
-  const bool is_dynamic_exe = ftype == FILE_TYPES::MH_EXECUTE && binary.has(LOAD_COMMAND_TYPES::LC_LOAD_DYLINKER);
+  const bool is_dynamic_exe =
+    ftype == Header::FILE_TYPE::EXECUTE && binary.has(LoadCommand::TYPE::LOAD_DYLINKER);
   if (dependents_count == 0 && is_dynamic_exe) {
       if (error) {
         *error = fmt::format(R"delim(
@@ -691,6 +700,66 @@ bool check_layout(const Binary& binary, std::string* error) {
   return true;
 }
 
+template<class MACHO_T>
+void foreach_segment_impl(BinaryStream& stream, const segment_callback_t cbk) {
+  using header_t = typename MACHO_T::header;
+  using segment_command_t = typename MACHO_T::segment_command;
+  auto res_hdr = stream.read<header_t>();
+  if (!res_hdr) {
+    return;
+  }
+  const auto& hdr = *res_hdr;
+
+  for (size_t i = 0; i < hdr.ncmds; ++i) {
+    const auto raw_cmd = stream.peek<details::load_command>();
+    if (!raw_cmd) {
+      break;
+    }
+    const auto cmd = LoadCommand::TYPE(raw_cmd->cmd);
+    const bool is_segment = cmd == LoadCommand::TYPE::SEGMENT ||
+                            cmd == LoadCommand::TYPE::SEGMENT_64;
+    if (is_segment) {
+      auto res_segment = stream.peek<segment_command_t>();
+      if (!res_segment) {
+        break;
+      }
+      std::string segname(res_segment->segname, 16);
+      cbk(segname, res_segment->fileoff, res_segment->filesize,
+          res_segment->vmaddr, res_segment->vmsize);
+    }
+
+    stream.increment_pos(raw_cmd->cmdsize);
+  }
+}
+
+void foreach_segment(BinaryStream& stream, const segment_callback_t cbk) {
+  ScopedStream scoped(stream);
+  auto magic_res = magic_from_stream(*scoped, /*keep_offset=*/true);
+  if (!magic_res) {
+    return;
+  }
+
+  const MACHO_TYPES magic = *magic_res;
+
+  if (magic == MACHO_TYPES::FAT_MAGIC || magic == MACHO_TYPES::FAT_CIGAM) {
+    LIEF_WARN("Can't get the file size of a FAT Macho-O");
+    return;
+  }
+
+  const bool is64 = magic == MACHO_TYPES::MH_MAGIC_64 ||
+                    magic == MACHO_TYPES::MH_CIGAM_64;
+
+  const bool is32 = magic == MACHO_TYPES::MH_MAGIC ||
+                    magic == MACHO_TYPES::MH_CIGAM;
+
+  if (!is64 && !is32) {
+    return;
+  }
+
+  return is64 ? foreach_segment_impl<details::MachO64>(*scoped, cbk) :
+                foreach_segment_impl<details::MachO32>(*scoped, cbk);
+}
+
 }
 }
 
