@@ -1,5 +1,5 @@
-/* Copyright 2017 - 2023 R. Thomas
- * Copyright 2017 - 2023 Quarkslab
+/* Copyright 2017 - 2024 R. Thomas
+ * Copyright 2017 - 2024 Quarkslab
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -19,6 +19,7 @@
 #include "LIEF/BinaryStream/BinaryStream.hpp"
 #include "LIEF/BinaryStream/SpanStream.hpp"
 #include "LIEF/utils.hpp"
+#include "internal_utils.hpp"
 
 #include "LIEF/PE/ResourcesManager.hpp"
 #include "LIEF/PE/ResourceData.hpp"
@@ -691,8 +692,7 @@ ok_error_t ResourcesParser::parse_tail_ext_dialog(ResourceDialog& dialog, Binary
   }
 
   if (auto res = stream.read<uint8_t>()) {
-    italic = *res;
-    dialog.italic_ = italic;
+    dialog.italic_ = static_cast<bool>(*res);
     LIEF_DEBUG("DLGTEMPLATEEX.italic: {}", italic);
   } else {
     LIEF_INFO("Can't read DLGTEMPLATEEX.italic");
